@@ -175,6 +175,7 @@ namespace xt
         using base_type = xstrided_container<self_type>;
         using semantic_base = xcontainer_semantic<self_type>;
         using container_type = typename base_type::container_type;
+        using allocator_type = typename base_type::allocator_type;
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
         using backstrides_type = typename base_type::backstrides_type;
@@ -405,7 +406,7 @@ namespace xt
         // in resize (called by assign), which is always true when dimension == 0.
         if (e.derived_cast().dimension() == 0)
         {
-            m_data.resize(1);
+            detail::resize_data_container(m_data, std::size_t(1));
         }
         semantic_base::assign(e);
     }
