@@ -38,6 +38,25 @@ namespace xt
 
 #define DL DEFAULT_LAYOUT
 
+    namespace detail
+    {
+        template <class T>
+        struct allocator_type_impl
+        {
+            using type = typename T::allocator_type;
+        };
+
+        template <class T, std::size_t N>
+        struct allocator_type_impl<std::array<T, N>>
+        {
+            using type = std::allocator<T>;  // fake allocator for testing
+        };
+    }
+
+    template <class T>
+    using allocator_type_t = typename detail::allocator_type_impl<T>::type;
+
+
     /**
      * @class xcontainer
      * @brief Base class for dense multidimensional containers.
@@ -58,9 +77,11 @@ namespace xt
 
         using inner_types = xcontainer_inner_types<D>;
         using container_type = typename inner_types::container_type;
-        using allocator_type = typename container_type::allocator_type;
+        using allocator_type = allocator_type_t<std::decay_t<container_type>>;
         using value_type = typename container_type::value_type;
-        using reference = typename container_type::reference;
+        using reference = std::conditional_t<std::is_const<container_type>::value,  
+                                             typename container_type::const_reference,
+                                             typename container_type::reference>;
         using const_reference = typename container_type::const_reference;
         using pointer = typename container_type::pointer;
         using const_pointer = typename container_type::const_pointer;
@@ -91,9 +112,9 @@ namespace xt
 
         constexpr size_type dimension() const noexcept;
 
-        const inner_shape_type& shape() const noexcept;
-        const inner_strides_type& strides() const noexcept;
-        const inner_backstrides_type& backstrides() const noexcept;
+        constexpr const inner_shape_type& shape() const noexcept;
+        constexpr const inner_strides_type& strides() const noexcept;
+        constexpr const inner_backstrides_type& backstrides() const noexcept;
 
         template <class... Args>
         reference operator()(Args... args);
@@ -311,6 +332,12 @@ namespace xt
         container_iterator data_xend(layout_type l) noexcept;
         const_container_iterator data_xend(layout_type l) const noexcept;
 
+    protected:
+
+        derived_type& derived_cast() & noexcept;
+        const derived_type& derived_cast() const & noexcept;
+        derived_type derived_cast() && noexcept;
+
     private:
 
         friend class xiterable<D>;
@@ -325,9 +352,6 @@ namespace xt
         inner_shape_type& mutable_shape();
         inner_strides_type& mutable_strides();
         inner_backstrides_type& mutable_backstrides();
-
-        derived_type& derived_cast();
-        const derived_type& derived_cast() const;
     };
 
 #undef DL
@@ -434,18 +458,6 @@ namespace xt
         return derived_cast().backstrides_impl();
     }
 
-    template <class D>
-    inline auto xcontainer<D>::derived_cast() -> derived_type&
-    {
-        return *static_cast<derived_type*>(this);
-    }
-
-    template <class D>
-    inline auto xcontainer<D>::derived_cast() const -> const derived_type&
-    {
-        return *static_cast<const derived_type*>(this);
-    }
-
     /**
      * @name Size and shape
      */
@@ -456,7 +468,7 @@ namespace xt
     template <class D>
     inline auto xcontainer<D>::size() const noexcept -> size_type
     {
-        return compute_size(shape());
+        return contiguous_layout ? data().size() : compute_size(shape());
     }
 
     /**
@@ -472,7 +484,7 @@ namespace xt
      * Returns the shape of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::shape() const noexcept -> const inner_shape_type&
+    constexpr inline auto xcontainer<D>::shape() const noexcept -> const inner_shape_type&
     {
         return derived_cast().shape_impl();
     }
@@ -481,7 +493,7 @@ namespace xt
      * Returns the strides of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::strides() const noexcept -> const inner_strides_type&
+    constexpr inline auto xcontainer<D>::strides() const noexcept -> const inner_strides_type&
     {
         return derived_cast().strides_impl();
     }
@@ -490,7 +502,7 @@ namespace xt
      * Returns the backstrides of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::backstrides() const noexcept -> const inner_backstrides_type&
+    constexpr inline auto xcontainer<D>::backstrides() const noexcept -> const inner_backstrides_type&
     {
         return derived_cast().backstrides_impl();
     }
@@ -1096,6 +1108,24 @@ namespace xt
         return data_xend_impl(data().end(), l);
     }
 
+    template <class D>
+    inline auto xcontainer<D>::derived_cast() & noexcept -> derived_type&
+    {
+        return *static_cast<derived_type*>(this);
+    }
+
+    template <class D>
+    inline auto xcontainer<D>::derived_cast() const & noexcept -> const derived_type&
+    {
+        return *static_cast<const derived_type*>(this);
+    }
+
+    template <class D>
+    inline auto xcontainer<D>::derived_cast() && noexcept -> derived_type
+    {
+        return *static_cast<derived_type*>(this);
+    }
+
     template <class D>
     inline auto xcontainer<D>::data_element(size_type i) -> reference
     {
@@ -1133,6 +1163,7 @@ namespace xt
         : base_type()
     {
         m_shape = xtl::make_sequence<inner_shape_type>(base_type::dimension(), 1);
+        m_strides = xtl::make_sequence<inner_strides_type>(base_type::dimension(), 0);
     }
 
     template <class D>
@@ -1194,12 +1225,14 @@ namespace xt
         template <class C, class S>
         inline void resize_data_container(C& c, S size)
         {
-            c.resize(size);
+            xt::resize_container(c, size);
         }
 
         template <class C, class S>
-        inline void resize_data_container(const C&, S)
+        inline void resize_data_container(const C& c, S size)
         {
+            (void)(c, size);  // remove unused parameter warning
+            XTENSOR_ASSERT_MSG(c.size() == size, "Trying to resize const data container with wrong size.");
         }
     }
 
@@ -1290,7 +1323,7 @@ namespace xt
         m_shape = xtl::forward_sequence<shape_type>(shape);
         resize_container(m_strides, m_shape.size());
         resize_container(m_backstrides, m_shape.size());
-        size_type data_size = compute_strides(m_shape, m_layout, m_strides, m_backstrides);
+        compute_strides(m_shape, m_layout, m_strides, m_backstrides);
     }
 }
 
