@@ -6,8 +6,8 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XEXCEPTION_HPP
-#define XEXCEPTION_HPP
+#ifndef XTENSOR_EXCEPTION_HPP
+#define XTENSOR_EXCEPTION_HPP
 
 #include <exception>
 #include <iterator>
@@ -71,7 +71,7 @@ namespace xt
     {
     public:
 
-        transpose_error(const std::string& msg);
+        explicit transpose_error(const std::string& msg);
 
         virtual const char* what() const noexcept;
 
@@ -104,13 +104,13 @@ namespace xt
 
     namespace detail
     {
-        template <class S, size_t dim>
+        template <class S, std::size_t dim>
         inline void check_index_impl(const S&)
         {
         }
 
-        template <class S, size_t dim, class... Args>
-        inline void check_index_impl(const S& shape, size_t i, Args... args)
+        template <class S, std::size_t dim, class... Args>
+        inline void check_index_impl(const S& shape, std::size_t arg, Args... args)
         {
             if (sizeof...(Args) + 1 > shape.size())
             {
@@ -118,9 +118,9 @@ namespace xt
             }
             else
             {
-                if (i >= shape[dim] && shape[dim] != 1)
+                if (arg >= shape[dim] && shape[dim] != 1)
                 {
-                    throw std::out_of_range("index " + std::to_string(i) + " is out of bounds for axis "
+                    throw std::out_of_range("index " + std::to_string(arg) + " is out of bounds for axis "
                         + std::to_string(dim) + " with size " + std::to_string(shape[dim]));
                 }
                 check_index_impl<S, dim + 1>(shape, args...);
@@ -139,7 +139,7 @@ namespace xt
     {
         auto dst = static_cast<typename S::size_type>(last - first);
         It efirst = last - std::min(shape.size(), dst);
-        size_t axis = 0;
+        std::size_t axis = 0;
         while (efirst != last)
         {
             if (*efirst >= shape[axis] && shape[axis] != 1)
@@ -151,20 +151,79 @@ namespace xt
         }
     }
 
+    /*******************
+     * check_dimension *
+     *******************/
+
+    template <class S, class... Args>
+    inline void check_dimension(const S& shape, Args...)
+    {
+        if (sizeof...(Args) > shape.size())
+        {
+            throw std::out_of_range("Number of arguments (" + std::to_string(sizeof...(Args)) + ") us greater "
+                + "than the number of dimensions (" + std::to_string(shape.size()) + ")");
+        }
+    }
+
+    /****************
+     * check_access *
+     ****************/
+
+    template <class S, class... Args>
+    inline void check_access(const S& shape, Args... args)
+    {
+        check_dimension(shape, args...);
+        check_index(shape, args...);
+    }
+
 #ifdef XTENSOR_ENABLE_ASSERT
-#define XTENSOR_ASSERT(expr) XTENSOR_ASSERT_IMPL(expr, __FILE__, __LINE__)
-#define XTENSOR_ASSERT_IMPL(expr, file, line)                                                                                    \
+#define XTENSOR_TRY(expr) XTENSOR_TRY_IMPL(expr, __FILE__, __LINE__)
+#define XTENSOR_TRY_IMPL(expr, file, line)                                                                                       \
     try                                                                                                                          \
     {                                                                                                                            \
         expr;                                                                                                                    \
     }                                                                                                                            \
-    catch (std::exception & e)                                                                                                   \
+    catch (std::exception& e)                                                                                                    \
     {                                                                                                                            \
-        throw std::runtime_error(std::string(file) + ':' + std::to_string(line)                                                  \
-            + ": check failed\n\t" + std::string(e.what()));                                                                     \
+        throw std::runtime_error(std::string(file) + ':' + std::to_string(line) + ": check failed\n\t" + std::string(e.what())); \
+    }
+#else
+#define XTENSOR_TRY(expr)
+#endif
+
+#ifdef XTENSOR_ENABLE_ASSERT
+#define XTENSOR_ASSERT(expr) XTENSOR_ASSERT_IMPL(expr, __FILE__, __LINE__)
+#define XTENSOR_ASSERT_IMPL(expr, file, line)                                                                                    \
+    if (!(expr))                                                                                                                 \
+    {                                                                                                                            \
+        throw std::runtime_error(std::string(file) + ':' + std::to_string(line) + ": assertion failed (" #expr ") \n\t");        \
     }
 #else
 #define XTENSOR_ASSERT(expr)
 #endif
-}
+
+#ifdef XTENSOR_ENABLE_CHECK_DIMENSION
+#define XTENSOR_CHECK_DIMENSION(S, ARGS) XTENSOR_TRY(check_dimension(S, ARGS))
+#else
+#define XTENSOR_CHECK_DIMENSION(S, ARGS)
 #endif
+
+#ifdef XTENSOR_ENABLE_ASSERT
+#define XTENSOR_ASSERT_MSG(expr, msg)                                                         \
+    if (!(expr))                                                                              \
+    {                                                                                         \
+        throw std::runtime_error(std::string("Assertion error!\n") + msg +                    \
+                                 "\n  " + __FILE__ + '(' + std::to_string(__LINE__) + ")\n"); \
+    }
+#else
+#define XTENSOR_ASSERT_MSG(expr, msg)
+#endif
+
+#define XTENSOR_PRECONDITION(expr, msg)                                                       \
+    if (!(expr))                                                                              \
+    {                                                                                         \
+        throw std::runtime_error(std::string("Precondition violation!\n") + msg +             \
+                                 "\n  " + __FILE__ + '(' + std::to_string(__LINE__) + ")\n"); \
+    }
+}
+#endif  // XEXCEPTION_HPP
