@@ -17,12 +17,11 @@
 #include "xtl/xtype_traits.hpp"
 
 #include "xutils.hpp"
+#include "xshape.hpp"
 
 namespace xt
 {
 
-    using xindex = std::vector<std::size_t>;
-
     /***************************
      * xexpression declaration *
      ***************************/
@@ -225,8 +224,17 @@ namespace xt
      * expression tag system *
      *************************/
 
-    struct xtensor_expression_tag {};
-    struct xoptional_expression_tag {};
+    struct xscalar_expression_tag
+    {
+    };
+
+    struct xtensor_expression_tag
+    {
+    };
+
+    struct xoptional_expression_tag
+    {
+    };
 
     namespace detail
     {
@@ -260,13 +268,31 @@ namespace xt
             using type = T;
         };
 
+        template <>
+        struct expression_tag_and<xscalar_expression_tag, xscalar_expression_tag>
+        {
+            using type = xscalar_expression_tag;
+        };
+
+        template <class T>
+        struct expression_tag_and<xscalar_expression_tag, T>
+        {
+            using type = T;
+        };
+
+        template <class T>
+        struct expression_tag_and<T, xscalar_expression_tag>
+            : expression_tag_and<xscalar_expression_tag, T>
+        {
+        };
+
         template <>
         struct expression_tag_and<xtensor_expression_tag, xoptional_expression_tag>
         {
             using type = xoptional_expression_tag;
         };
 
-        template<>
+        template <>
         struct expression_tag_and<xoptional_expression_tag, xtensor_expression_tag>
             : expression_tag_and<xtensor_expression_tag, xoptional_expression_tag>
         {
@@ -285,11 +311,33 @@ namespace xt
     template <class... T>
     struct xexpression_tag
     {
-        using type = detail::expression_tag_and_t<detail::get_expression_tag_t<T>...>;
+        using type = detail::expression_tag_and_t<detail::get_expression_tag_t<std::decay_t<const_xclosure_t<T>>>...>;
     };
 
     template <class... T>
     using xexpression_tag_t = typename xexpression_tag<T...>::type;
+
+    template <class E>
+    struct is_xtensor_expression : std::is_same<xexpression_tag_t<E>, xtensor_expression_tag>
+    {
+    };
+
+    template <class E>
+    struct is_xoptional_expression : std::is_same<xexpression_tag_t<E>, xoptional_expression_tag>
+    {
+    };
+
+    /********************************
+     * xoptional_comparable concept *
+     ********************************/
+
+    template <class... E>
+    struct xoptional_comparable : xtl::conjunction<xtl::disjunction<is_xtensor_expression<E>,
+                                                                    is_xoptional_expression<E>
+                                                                   >...
+                                                  >
+    {
+    };
 }
 
 #endif
